@@ -26,14 +26,14 @@
  *
  * -------------------------------------------------------------------------- */
 
-#include "core/kernelAudio.h"
-#include "core/const.h"
-#include "core/model/kernelAudio.h"
-#include "core/model/model.h"
-#include "deps/mcl-audio-buffer/src/audioBuffer.hpp"
-#include "utils/log.h"
-#include "utils/string.h"
-#include "utils/vector.h"
+#include "src/core/kernelAudio.h"
+#include "src/core/const.h"
+#include "src/core/model/kernelAudio.h"
+#include "src/core/model/model.h"
+#include "src/deps/mcl-audio-buffer/src/audioBuffer.hpp"
+#include "src/utils/log.h"
+#include "src/utils/string.h"
+#include "src/utils/vector.h"
 #include <cassert>
 #include <cstddef>
 
@@ -59,6 +59,7 @@ KernelAudio::KernelAudio(model::Model& model)
 , onStreamAboutToOpen(nullptr)
 , onStreamOpened(nullptr)
 , m_model(model)
+, m_jackMaxOutputChannels(0)
 {
 }
 
@@ -202,8 +203,9 @@ KernelAudio::Device KernelAudio::getCurrentOutDevice() const
 	const model::KernelAudio& kernelAudio = m_model.get().kernelAudio;
 
 	Device d        = fetchDevice(m_rtAudio->isStreamOpen() ? kernelAudio.deviceOut.id : m_rtAudio->getDefaultOutputDevice());
-	d.channelsCount = kernelAudio.deviceOut.channelsCount;
 	d.channelsStart = kernelAudio.deviceOut.channelsStart;
+	if (getAPI() == RtAudio::Api::UNIX_JACK)
+		d.maxOutputChannels = m_jackMaxOutputChannels;
 
 	return d;
 }
@@ -215,7 +217,6 @@ KernelAudio::Device KernelAudio::getCurrentInDevice() const
 	const model::KernelAudio& kernelAudio = m_model.get().kernelAudio;
 
 	Device d        = fetchDevice(m_rtAudio->isStreamOpen() ? kernelAudio.deviceIn.id : m_rtAudio->getDefaultInputDevice());
-	d.channelsCount = kernelAudio.deviceIn.channelsCount;
 	d.channelsStart = kernelAudio.deviceIn.channelsStart;
 
 	return d;
@@ -272,7 +273,6 @@ m::KernelAudio::Device KernelAudio::fetchDevice(unsigned int deviceId) const
 	    info.isDefaultOutput,
 	    info.isDefaultInput,
 	    0,
-	    0,
 	    info.sampleRates};
 }
 
@@ -358,6 +358,11 @@ KernelAudio::OpenStreamResult KernelAudio::openStream_(
 	outParams.nChannels    = out.channelsCount;
 	outParams.firstChannel = out.channelsStart;
 
+	/* Store the current number of output channels for JACK. This is needed when calling getCurrentOutDevice(),
+	where the maxOutputChannels parameter is not correctly updated for JACK. */
+
+	m_jackMaxOutputChannels = out.channelsCount;
+
 	if (in.id != 0)
 	{
 		inParams.deviceId     = in.id == -1 ? m_rtAudio->getDefaultInputDevice() : in.id;
