@@ -24,11 +24,11 @@
  *
  * -------------------------------------------------------------------------- */
 
-#include "core/mixer.h"
-#include "core/const.h"
-#include "core/model/model.h"
-#include "utils/log.h"
-#include "utils/math.h"
+#include "src/core/mixer.h"
+#include "src/core/const.h"
+#include "src/core/model/model.h"
+#include "src/utils/log.h"
+#include "src/utils/math.h"
 
 namespace giada::m
 {
@@ -140,7 +140,7 @@ void Mixer::render(const mcl::AudioBuffer& in, const model::Document& document_R
 	const model::Tracks&      tracks      = document_RT.tracks;
 	const model::KernelAudio& kernelAudio = document_RT.kernelAudio;
 
-	const Channel& masterInCh = tracks.getChannel(Mixer::MASTER_IN_CHANNEL_ID);
+	const Channel& masterInCh = tracks.getChannel(MASTER_IN_CHANNEL_ID);
 
 	const bool  hasInput        = in.isAllocd();
 	const bool  seqIsActive     = sequencer.isActive();
@@ -279,7 +279,7 @@ int Mixer::lineInRec(const mcl::AudioBuffer& inBuf, mcl::AudioBuffer& recBuf, Fr
 	const Frame srcOffset    = 0;
 	const Frame destOffset   = inputTracker % maxFrames; // loop over at maxFrames
 
-	recBuf.sum(inBuf, framesToCopy, srcOffset, destOffset, inVol);
+	recBuf.sumAll(inBuf, framesToCopy, srcOffset, destOffset, inVol);
 
 	return inputTracker + inBuf.countFrames();
 }
@@ -294,7 +294,7 @@ void Mixer::processLineIn(const model::Mixer& mixer, const mcl::AudioBuffer& inB
 
 	assert(inBuf.countChannels() <= mixer.getInBuffer().countChannels());
 
-	mixer.getInBuffer().set(inBuf, inVol);
+	mixer.getInBuffer().setAll(inBuf, inVol);
 
 	const Peak peak = makePeak(mixer.getInBuffer());
 
@@ -323,13 +323,18 @@ void Mixer::finalizeOutput(const model::Mixer& mixer, mcl::AudioBuffer& buf,
     bool inToOut, bool shouldLimit, float vol) const
 {
 	if (inToOut)
-		buf.sum(mixer.getInBuffer(), vol);
+		buf.sumAll(mixer.getInBuffer(), vol);
 	else
 		buf.applyGain(vol);
 
 	if (shouldLimit)
 		limit(buf);
+}
 
+/* -------------------------------------------------------------------------- */
+
+void Mixer::updateOutputPeak(const model::Mixer& mixer, const mcl::AudioBuffer& buf) const
+{
 	mixer.a_setPeakOut({buf.getPeak(CH_LEFT), buf.getPeak(CH_RIGHT)});
 }
 } // namespace giada::m
