@@ -24,11 +24,12 @@
  *
  * -------------------------------------------------------------------------- */
 
-#include "core/patchFactory.h"
-#include "core/const.h"
-#include "core/mixer.h"
-#include "utils/fs.h"
-#include "utils/log.h"
+#include "src/core/patchFactory.h"
+#include "src/core/const.h"
+#include "src/core/mixer.h"
+#include "src/gui/const.h"
+#include "src/utils/fs.h"
+#include "src/utils/log.h"
 #include <fstream>
 #include <nlohmann/json.hpp>
 
@@ -36,6 +37,83 @@ namespace giada::m::patchFactory
 {
 namespace
 {
+constexpr auto PATCH_KEY_HEADER                       = "header";
+constexpr auto PATCH_KEY_VERSION_MAJOR                = "version_major";
+constexpr auto PATCH_KEY_VERSION_MINOR                = "version_minor";
+constexpr auto PATCH_KEY_VERSION_PATCH                = "version_patch";
+constexpr auto PATCH_KEY_NAME                         = "name";
+constexpr auto PATCH_KEY_BPM                          = "bpm";
+constexpr auto PATCH_KEY_BARS                         = "bars";
+constexpr auto PATCH_KEY_BEATS                        = "beats";
+constexpr auto PATCH_KEY_QUANTIZE                     = "quantize";
+constexpr auto PATCH_KEY_METRONOME                    = "metronome";
+constexpr auto PATCH_KEY_SAMPLERATE                   = "samplerate";
+constexpr auto PATCH_KEY_TRACKS                       = "tracks";
+constexpr auto PATCH_KEY_PLUGINS                      = "plugins";
+constexpr auto PATCH_KEY_CHANNELS                     = "channels";
+constexpr auto PATCH_KEY_CHANNEL_TYPE                 = "type";
+constexpr auto PATCH_KEY_CHANNEL_ID                   = "id";
+constexpr auto PATCH_KEY_CHANNEL_SIZE                 = "size";
+constexpr auto PATCH_KEY_CHANNEL_NAME                 = "name";
+constexpr auto PATCH_KEY_CHANNEL_MUTE                 = "mute";
+constexpr auto PATCH_KEY_CHANNEL_SOLO                 = "solo";
+constexpr auto PATCH_KEY_CHANNEL_VOLUME               = "volume";
+constexpr auto PATCH_KEY_CHANNEL_PAN                  = "pan";
+constexpr auto PATCH_KEY_CHANNEL_SEND_TO_MASTER       = "send_to_master";
+constexpr auto PATCH_KEY_CHANNEL_EXTRA_OUTPUTS        = "extra_outputs";
+constexpr auto PATCH_KEY_CHANNEL_MIDI_IN              = "midi_in";
+constexpr auto PATCH_KEY_CHANNEL_MIDI_IN_VELO_AS_VOL  = "midi_in_velo_as_vol";
+constexpr auto PATCH_KEY_CHANNEL_MIDI_IN_KEYPRESS     = "midi_in_keypress";
+constexpr auto PATCH_KEY_CHANNEL_MIDI_IN_KEYREL       = "midi_in_keyrel";
+constexpr auto PATCH_KEY_CHANNEL_MIDI_IN_KILL         = "midi_in_kill";
+constexpr auto PATCH_KEY_CHANNEL_MIDI_IN_ARM          = "midi_in_arm";
+constexpr auto PATCH_KEY_CHANNEL_MIDI_IN_VOLUME       = "midi_in_volume";
+constexpr auto PATCH_KEY_CHANNEL_MIDI_IN_MUTE         = "midi_in_mute";
+constexpr auto PATCH_KEY_CHANNEL_MIDI_IN_FILTER       = "midi_in_filter";
+constexpr auto PATCH_KEY_CHANNEL_MIDI_IN_SOLO         = "midi_in_solo";
+constexpr auto PATCH_KEY_CHANNEL_MIDI_OUT_L           = "midi_out_l";
+constexpr auto PATCH_KEY_CHANNEL_MIDI_OUT_L_PLAYING   = "midi_out_l_playing";
+constexpr auto PATCH_KEY_CHANNEL_MIDI_OUT_L_MUTE      = "midi_out_l_mute";
+constexpr auto PATCH_KEY_CHANNEL_MIDI_OUT_L_SOLO      = "midi_out_l_solo";
+constexpr auto PATCH_KEY_CHANNEL_WAVE_ID              = "wave_id";
+constexpr auto PATCH_KEY_CHANNEL_KEY                  = "key";
+constexpr auto PATCH_KEY_CHANNEL_MODE                 = "mode";
+constexpr auto PATCH_KEY_CHANNEL_BEGIN                = "begin";
+constexpr auto PATCH_KEY_CHANNEL_END                  = "end";
+constexpr auto PATCH_KEY_CHANNEL_SHIFT                = "shift";
+constexpr auto PATCH_KEY_CHANNEL_HAS_ACTIONS          = "has_actions";
+constexpr auto PATCH_KEY_CHANNEL_READ_ACTIONS         = "read_actions";
+constexpr auto PATCH_KEY_CHANNEL_PITCH                = "pitch";
+constexpr auto PATCH_KEY_CHANNEL_INPUT_MONITOR        = "input_monitor";
+constexpr auto PATCH_KEY_CHANNEL_OVERDUB_PROTECTION   = "overdub_protection";
+constexpr auto PATCH_KEY_CHANNEL_MIDI_IN_READ_ACTIONS = "midi_in_read_actions";
+constexpr auto PATCH_KEY_CHANNEL_MIDI_IN_PITCH        = "midi_in_pitch";
+constexpr auto PATCH_KEY_CHANNEL_MIDI_OUT             = "midi_out";
+constexpr auto PATCH_KEY_CHANNEL_MIDI_OUT_CHAN        = "midi_out_chan";
+constexpr auto PATCH_KEY_CHANNEL_PLUGINS              = "plugins";
+constexpr auto PATCH_KEY_CHANNEL_ARMED                = "armed";
+constexpr auto PATCH_KEY_WAVES                        = "waves";
+constexpr auto PATCH_KEY_WAVE_ID                      = "id";
+constexpr auto PATCH_KEY_WAVE_PATH                    = "path";
+constexpr auto PATCH_KEY_ACTIONS                      = "actions";
+constexpr auto PATCH_KEY_PLUGIN_ID                    = "id";
+constexpr auto PATCH_KEY_PLUGIN_JUCE_ID               = "juce_id";
+constexpr auto PATCH_KEY_PLUGIN_JUCE_ID_deprecated    = "path";
+constexpr auto PATCH_KEY_PLUGIN_BYPASS                = "bypass";
+constexpr auto PATCH_KEY_PLUGIN_STATE                 = "state";
+constexpr auto PATCH_KEY_PLUGIN_MIDI_IN_PARAMS        = "midi_in_params";
+constexpr auto PATCH_KEY_TRACK_WIDTH                  = "width";
+constexpr auto PATCH_KEY_TRACK_INTERNAL               = "internal";
+constexpr auto PATCH_KEY_TRACK_CHANNELS               = "channels";
+constexpr auto G_PATCH_KEY_ACTION_ID                  = "id";
+constexpr auto G_PATCH_KEY_ACTION_CHANNEL             = "channel";
+constexpr auto G_PATCH_KEY_ACTION_FRAME               = "frame";
+constexpr auto G_PATCH_KEY_ACTION_EVENT               = "event";
+constexpr auto G_PATCH_KEY_ACTION_PREV                = "prev";
+constexpr auto G_PATCH_KEY_ACTION_NEXT                = "next";
+
+/* -------------------------------------------------------------------------- */
+
 void readCommons_(Patch& patch, const nlohmann::json& j)
 {
 	patch.name       = j.value(PATCH_KEY_NAME, G_DEFAULT_PATCH_NAME);
@@ -79,14 +157,12 @@ void readPlugins_(Patch& patch, const nlohmann::json& j)
 	{
 		Patch::Plugin p;
 		p.id     = jplugin.value(PATCH_KEY_PLUGIN_ID, ++id);
-		p.path   = jplugin.value(PATCH_KEY_PLUGIN_PATH, "");
+		p.juceId = jplugin.value(PATCH_KEY_PLUGIN_JUCE_ID, "");
 		p.bypass = jplugin.value(PATCH_KEY_PLUGIN_BYPASS, false);
-
-		if (patch.version < Patch::Version{0, 17, 0})
-			for (const auto& jparam : jplugin[PATCH_KEY_PLUGIN_PARAMS])
-				p.params.push_back(jparam);
-		else
-			p.state = jplugin.value(PATCH_KEY_PLUGIN_STATE, "");
+		p.state  = jplugin.value(PATCH_KEY_PLUGIN_STATE, "");
+		/* Patches < 1.3.0 have the deprecated JUCE id for plug-ins. */
+		if (patch.version < Version{1, 3, 0})
+			p.juceId = jplugin.value(PATCH_KEY_PLUGIN_JUCE_ID_deprecated, "");
 
 		for (const auto& jmidiParam : jplugin[PATCH_KEY_PLUGIN_MIDI_IN_PARAMS])
 			p.midiInParams.push_back(jmidiParam);
@@ -140,7 +216,7 @@ void readChannels_(Patch& patch, const nlohmann::json& j)
 	if (!j.contains(PATCH_KEY_CHANNELS))
 		return;
 
-	ID defaultId = Mixer::PREVIEW_CHANNEL_ID;
+	ID defaultId = PREVIEW_CHANNEL_ID;
 
 	for (const auto& jchannel : j[PATCH_KEY_CHANNELS])
 	{
@@ -155,6 +231,7 @@ void readChannels_(Patch& patch, const nlohmann::json& j)
 		c.solo              = jchannel.value(PATCH_KEY_CHANNEL_SOLO, 0);
 		c.pan               = jchannel.value(PATCH_KEY_CHANNEL_PAN, 0.5f);
 		c.hasActions        = jchannel.value(PATCH_KEY_CHANNEL_HAS_ACTIONS, false);
+		c.sendToMaster      = jchannel.value(PATCH_KEY_CHANNEL_SEND_TO_MASTER, true);
 		c.midiIn            = jchannel.value(PATCH_KEY_CHANNEL_MIDI_IN, 0);
 		c.midiInKeyPress    = jchannel.value(PATCH_KEY_CHANNEL_MIDI_IN_KEYPRESS, 0);
 		c.midiInKeyRel      = jchannel.value(PATCH_KEY_CHANNEL_MIDI_IN_KEYREL, 0);
@@ -171,8 +248,8 @@ void readChannels_(Patch& patch, const nlohmann::json& j)
 		c.armed             = jchannel.value(PATCH_KEY_CHANNEL_ARMED, false);
 		c.mode              = static_cast<SamplePlayerMode>(jchannel.value(PATCH_KEY_CHANNEL_MODE, 1));
 		c.waveId            = jchannel.value(PATCH_KEY_CHANNEL_WAVE_ID, 0);
-		c.begin             = jchannel.value(PATCH_KEY_CHANNEL_BEGIN, 0);
-		c.end               = jchannel.value(PATCH_KEY_CHANNEL_END, 0);
+		c.range.a           = jchannel.value(PATCH_KEY_CHANNEL_BEGIN, 0);
+		c.range.b           = jchannel.value(PATCH_KEY_CHANNEL_END, 0);
 		c.shift             = jchannel.value(PATCH_KEY_CHANNEL_SHIFT, 0);
 		c.readActions       = jchannel.value(PATCH_KEY_CHANNEL_READ_ACTIONS, false);
 		c.pitch             = jchannel.value(PATCH_KEY_CHANNEL_PITCH, G_DEFAULT_PITCH);
@@ -188,6 +265,10 @@ void readChannels_(Patch& patch, const nlohmann::json& j)
 			for (const auto& jplugin : jchannel[PATCH_KEY_CHANNEL_PLUGINS])
 				c.pluginIds.push_back(jplugin);
 
+		if (jchannel.contains(PATCH_KEY_CHANNEL_EXTRA_OUTPUTS))
+			for (const auto& joutput : jchannel[PATCH_KEY_CHANNEL_EXTRA_OUTPUTS])
+				c.extraOutputs.push_back(joutput);
+
 		patch.channels.push_back(c);
 	}
 }
@@ -202,10 +283,10 @@ void writePlugins_(const Patch& patch, nlohmann::json& j)
 	{
 		nlohmann::json jplugin;
 
-		jplugin[PATCH_KEY_PLUGIN_ID]     = p.id;
-		jplugin[PATCH_KEY_PLUGIN_PATH]   = p.path;
-		jplugin[PATCH_KEY_PLUGIN_BYPASS] = p.bypass;
-		jplugin[PATCH_KEY_PLUGIN_STATE]  = p.state;
+		jplugin[PATCH_KEY_PLUGIN_ID]      = p.id;
+		jplugin[PATCH_KEY_PLUGIN_JUCE_ID] = p.juceId;
+		jplugin[PATCH_KEY_PLUGIN_BYPASS]  = p.bypass;
+		jplugin[PATCH_KEY_PLUGIN_STATE]   = p.state;
 
 		jplugin[PATCH_KEY_PLUGIN_MIDI_IN_PARAMS] = nlohmann::json::array();
 		for (uint32_t param : p.midiInParams)
@@ -274,9 +355,9 @@ void writeWaves_(const Patch& patch, nlohmann::json& j)
 void writeCommons_(const Patch& patch, nlohmann::json& j)
 {
 	j[PATCH_KEY_HEADER]        = "GIADAPTC";
-	j[PATCH_KEY_VERSION_MAJOR] = G_VERSION_MAJOR;
-	j[PATCH_KEY_VERSION_MINOR] = G_VERSION_MINOR;
-	j[PATCH_KEY_VERSION_PATCH] = G_VERSION_PATCH;
+	j[PATCH_KEY_VERSION_MAJOR] = G_VERSION.getMajor();
+	j[PATCH_KEY_VERSION_MINOR] = G_VERSION.getMinor();
+	j[PATCH_KEY_VERSION_PATCH] = G_VERSION.getPatch();
 	j[PATCH_KEY_NAME]          = patch.name;
 	j[PATCH_KEY_BARS]          = patch.bars;
 	j[PATCH_KEY_BEATS]         = patch.beats;
@@ -306,6 +387,7 @@ void writeChannels_(const Patch& patch, nlohmann::json& j)
 		jchannel[PATCH_KEY_CHANNEL_PAN]                  = c.pan;
 		jchannel[PATCH_KEY_CHANNEL_HAS_ACTIONS]          = c.hasActions;
 		jchannel[PATCH_KEY_CHANNEL_ARMED]                = c.armed;
+		jchannel[PATCH_KEY_CHANNEL_SEND_TO_MASTER]       = c.sendToMaster;
 		jchannel[PATCH_KEY_CHANNEL_MIDI_IN]              = c.midiIn;
 		jchannel[PATCH_KEY_CHANNEL_MIDI_IN_KEYREL]       = c.midiInKeyRel;
 		jchannel[PATCH_KEY_CHANNEL_MIDI_IN_KEYPRESS]     = c.midiInKeyPress;
@@ -322,8 +404,8 @@ void writeChannels_(const Patch& patch, nlohmann::json& j)
 		jchannel[PATCH_KEY_CHANNEL_KEY]                  = c.key;
 		jchannel[PATCH_KEY_CHANNEL_WAVE_ID]              = c.waveId;
 		jchannel[PATCH_KEY_CHANNEL_MODE]                 = static_cast<int>(c.mode);
-		jchannel[PATCH_KEY_CHANNEL_BEGIN]                = c.begin;
-		jchannel[PATCH_KEY_CHANNEL_END]                  = c.end;
+		jchannel[PATCH_KEY_CHANNEL_BEGIN]                = c.range.a;
+		jchannel[PATCH_KEY_CHANNEL_END]                  = c.range.b;
 		jchannel[PATCH_KEY_CHANNEL_SHIFT]                = c.shift;
 		jchannel[PATCH_KEY_CHANNEL_READ_ACTIONS]         = c.readActions;
 		jchannel[PATCH_KEY_CHANNEL_PITCH]                = c.pitch;
@@ -339,6 +421,10 @@ void writeChannels_(const Patch& patch, nlohmann::json& j)
 		for (ID pid : c.pluginIds)
 			jchannel[PATCH_KEY_CHANNEL_PLUGINS].push_back(pid);
 
+		jchannel[PATCH_KEY_CHANNEL_EXTRA_OUTPUTS] = nlohmann::json::array();
+		for (int output : c.extraOutputs)
+			jchannel[PATCH_KEY_CHANNEL_EXTRA_OUTPUTS].push_back(output);
+
 		j[PATCH_KEY_CHANNELS].push_back(jchannel);
 	}
 }
@@ -350,7 +436,7 @@ void modernize_(Patch& patch)
 	/* 1.1.0
 	Older patches don't contain Tracks. Let's recreate a basic Track layout: one
 	internal, one external. */
-	if (patch.version < Patch::Version{1, 1, 0})
+	if (patch.version < Version{1, 1, 0})
 	{
 		const ID groupChannelId = 8192;
 		patch.tracks.push_back({G_DEFAULT_TRACK_WIDTH, /*internal=*/true, {}});
@@ -365,9 +451,9 @@ void modernize_(Patch& patch)
 		/* 0.16.3
 		Make sure that ChannelType is correct: ID 1, 2 are MASTER channels, ID 3
 		is PREVIEW channel. */
-		if (c.id == Mixer::MASTER_OUT_CHANNEL_ID || c.id == Mixer::MASTER_IN_CHANNEL_ID)
+		if (c.id == MASTER_OUT_CHANNEL_ID || c.id == MASTER_IN_CHANNEL_ID)
 			c.type = ChannelType::MASTER;
-		else if (c.id == Mixer::PREVIEW_CHANNEL_ID)
+		else if (c.id == PREVIEW_CHANNEL_ID)
 			c.type = ChannelType::PREVIEW;
 
 		/* 0.16.4
@@ -385,7 +471,7 @@ void modernize_(Patch& patch)
 
 		/* 1.1.0
 		Let's put non-group channels into the relevant tracks. */
-		if (patch.version < Patch::Version{1, 1, 0} && c.type != ChannelType::GROUP)
+		if (patch.version < Version{1, 1, 0} && c.type != ChannelType::GROUP)
 		{
 			const std::size_t targetIndex = isInternalChannel ? 0 : 1;
 			patch.tracks[targetIndex].channels.push_back(c.id);
@@ -444,7 +530,7 @@ Patch deserialize(const std::string& filePath)
 		    static_cast<int>(j[PATCH_KEY_VERSION_MAJOR]),
 		    static_cast<int>(j[PATCH_KEY_VERSION_MINOR]),
 		    static_cast<int>(j[PATCH_KEY_VERSION_PATCH])};
-		if (patch.version < Patch::Version{0, 16, 0})
+		if (patch.version < Version{0, 16, 0})
 		{
 			patch.status = G_FILE_UNSUPPORTED;
 			return patch;
