@@ -27,7 +27,7 @@
 #ifndef GD_MIDI_OUTPUT_SAMPLE_CH_H
 #define GD_MIDI_OUTPUT_SAMPLE_CH_H
 
-#include "midiOutputBase.h"
+#include "src/gui/dialogs/midiIO/midiOutputBase.h"
 
 namespace giada::v
 {
