@@ -27,7 +27,7 @@
 #ifndef G_LANGMAPPER_H
 #define G_LANGMAPPER_H
 
-#include "mapper.h"
+#include "src/mapper.h"
 #include <nlohmann/json.hpp>
 #include <string>
 #include <unordered_map>
@@ -222,7 +222,7 @@ public:
 	static constexpr auto SAMPLEEDITOR_TOOLS_FADE_IN        = "sampleEditor_tools_fadeIn";
 	static constexpr auto SAMPLEEDITOR_TOOLS_FADE_OUT       = "sampleEditor_tools_fadeOut";
 	static constexpr auto SAMPLEEDITOR_TOOLS_SMOOTH_EDGES   = "sampleEditor_tools_smoothEdgdes";
-	static constexpr auto SAMPLEEDITOR_TOOLS_SET_BEGIN_END  = "sampleEditor_tools_setBeginEnd";
+	static constexpr auto SAMPLEEDITOR_TOOLS_SET_RANGE      = "sampleEditor_tools_setRange";
 	static constexpr auto SAMPLEEDITOR_TOOLS_TO_NEW_CHANNEL = "sampleEditor_tools_toNewChannel";
 
 	static constexpr auto ACTIONEDITOR_TITLE            = "actionEditor_title";
@@ -284,6 +284,7 @@ public:
 	static constexpr auto CONFIG_AUDIO_TITLE                 = "config_audio_title";
 	static constexpr auto CONFIG_AUDIO_SYSTEM                = "config_audio_system";
 	static constexpr auto CONFIG_AUDIO_BUFFERSIZE            = "config_audio_bufferSize";
+	static constexpr auto CONFIG_AUDIO_TOTALOUTPUTCHANNELS   = "config_audio_totalOutputChannels";
 	static constexpr auto CONFIG_AUDIO_SAMPLERATE            = "config_audio_sampleRate";
 	static constexpr auto CONFIG_AUDIO_OUTPUTDEVICE          = "config_audio_outputDevice";
 	static constexpr auto CONFIG_AUDIO_OUTPUTCHANNELS        = "config_audio_outputChannels";
@@ -342,7 +343,12 @@ public:
 	static constexpr auto CONFIG_PLUGINS_SCAN        = "config_plugins_scan";
 	static constexpr auto CONFIG_PLUGINS_INVALIDPATH = "config_plugins_invalidPath";
 
-	static constexpr auto CHANNELROUTING_TITLE = "channelRouting_title";
+	static constexpr auto CHANNELROUTING_TITLE              = "channelRouting_title";
+	static constexpr auto CHANNELROUTING_SENDTOMASTEROUT    = "channelRouting_sendToMasterOut";
+	static constexpr auto CHANNELROUTING_SENDTOPARENTGROUP  = "channelRouting_sendToParentGroup";
+	static constexpr auto CHANNELROUTING_ADDNEWOUTPUT       = "channelRouting_addNewOutput";
+	static constexpr auto CHANNELROUTING_OUTPUT_CHOOSE      = "channelRouting_output_choose";
+	static constexpr auto CHANNELROUTING_OUTPUT_UNREACHABLE = "channelRouting_output_unreachable";
 
 	LangMap();
 
